--------------------------------------------------------
--
-- This is the dcc top module
--
--	6/17/05 change to DCCspec. 6/1/05 as version 2b01
---------------------------------------------------------

library ieee,mylib;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use mylib.vtx_pack.all;
use mylib.dcc_pack.all;

entity dcc is
    port (
	        reset_dll : in std_logic;
	        ttc_pads : inout ttc_pad;
			  clk_pads : inout clk_pad;
			  pci1_pads : inout lrb_pad;
			  pci2_pads : inout lrb_pad;
			  pci3_pads : inout pci3_pad;
			  slink_pads : inout slink_pad;
			  ram_pads : inout ram_pad;
			  led_pads : inout led_pad;
			  lvds_pads : out lvds_pad
	 );
end dcc;

architecture dcc_arch of dcc is
constant version : std_logic_vector(15 downto 0) :=  x"2c2b"; -- version number
-- fast monitoring signal 
signal state: std_logic_vector(3 downto 0) := (others => '0');
--- clocks signals
signal mclk: std_logic := '0';
signal mclkn: std_logic := '0';
signal mclk90: std_logic := '0';
signal mclk270: std_logic := '0';
signal ttc_clk: std_logic := '0';
signal lock: std_logic_vector(3 downto 0) := (others => '0');
signal not_locked : std_logic := '0';
-- pci3 output signals
signal n_strobe: std_logic := '0';
signal rst_ttc: std_logic := '0';
signal wr_ttc_id: std_logic := '0';
signal rd_ttc_id: std_logic := '0';
signal wr_bcn_off: std_logic := '0';
signal rd_bcn_off: std_logic := '0';
signal wr_i2c: std_logic := '0';
signal rd_i2c: std_logic := '0';
signal wr_scale: std_logic := '0';
signal rd_scale: std_logic := '0';
signal wr_timeout: std_logic := '0';
signal rd_timeout: std_logic := '0';
signal wr_src_id: std_logic := '0';
signal rd_src_id: std_logic := '0';
signal rd_mon_wc: std_logic := '0';
signal rd_status: std_logic := '0';
signal cache_chk: std_logic := '0';
signal cache_hit: std_logic := '0';
signal flush_cache: std_logic := '0';
signal rd_ddr_pa: std_logic := '0';
signal ld_ddr_pa: std_logic := '0';
signal rst_ddr_pa: std_logic := '0';
signal inc_ddr_pa: std_logic := '0';
signal ddr_pa: std_logic_vector(6 downto 0) := (others => '0');
signal ddr_pa_off: std_logic_vector(6 downto 0) := (others => '0');
signal pa: std_logic_vector(31 downto 0) := (others => '0');
signal w_addr: std_logic_vector(11 downto 0) := (others => '0');
signal rd_evn_pntr: std_logic := '0';
signal rd_evn_buf: std_logic := '0';
signal rd_lrb_en: std_logic := '0';
signal pci3_LRB_stat: std_logic := '0';
signal monitor_start: std_logic := '0';
signal pci1_mon_ready: std_logic := '0';
signal pci2_mon_ready: std_logic := '0';
signal rd_lrb_mon_pci1: std_logic := '0';
signal rd_lrb_mon_pci2: std_logic := '0';
signal wr_slink_data: std_logic := '0';
signal rd_slink_pntr: std_logic := '0';
-- dcc output signals
signal auto_monitor: std_logic := '0';
signal lrb_en: std_logic_vector(31 downto 0) := (others => '0');
signal lrb_enp: std_logic_vector(17 downto 0) := (others => '0');
-- ttcrx output signals
signal event_number_avl: std_logic := '0';
signal event_number: std_logic_vector(71 downto 0) := (others => '0');
signal en_brcst: std_logic := '0';
signal ttc_start: std_logic := '0';
signal ttc_stop: std_logic := '0';
signal ttc_soft_reset: std_logic := '0';
signal ttc_LRB_stat: std_logic := '0';
signal ttc_ready: std_logic := '0';
signal ttc_serr: std_logic := '0';
signal ttc_derr: std_logic := '0';
signal inc_oc: std_logic := '0';
signal inc_l1ac: std_logic := '0';
signal inc_bcnterr: std_logic := '0';
signal inc_serr: std_logic := '0';
signal inc_derr: std_logic := '0';
signal inc_rate_OFW: std_logic := '0';
signal inc_slink_wc: std_logic := '0';
signal inc_slink_bc: std_logic := '0';
signal inc_mon_cntr: std_logic := '0';
signal ttc_bcnt_err: std_logic := '0';
signal rate_OFW: std_logic := '0';
signal rate_OFW_q: std_logic := '0';
signal rate_OFWp: std_logic := '0';
signal sync_lost: std_logic := '0';
signal ovfl_warning: std_logic := '0';
signal evn_fifo_full: std_logic := '0';
--
signal ttc_soft_resetp: std_logic := '0';
signal ttc_resync: std_logic := '0';
signal oc_cntr: std_logic_vector(3 downto 0) := (others => '0');
signal resync_done: std_logic := '0';
signal dcc_quiet: std_logic := '0';
-- pci3 signals
signal r_data: std_logic_vector(31 downto 0) := (others => '0');
signal w_data: std_logic_vector(31 downto 0) := (others => '0');
signal addr: std_logic_vector(13 downto 0) := (others => '0');
signal strobe: std_logic := '0';
signal rd_config: std_logic := '0';
signal wr_config: std_logic := '0';
signal is_high: std_logic := '0';
signal wr_ddr_data: std_logic := '0';
signal rd_ddr_data: std_logic := '0';
signal rd_slink_data: std_logic := '0';
signal rd_CRC_error: std_logic := '0';
signal rd_lrb_sgl_err: std_logic := '0';
signal rd_lrb_dbl_err: std_logic := '0';
signal rd_evn_mm: std_logic := '0';
signal rd_counters: std_logic := '0';
signal rd_status_cntr: std_logic := '0';
signal pci3_cycle: std_logic := '0';
signal reset: std_logic := '0';
signal reset_err: std_logic := '0';
-- dcc signals
signal run: std_logic := '0';
signal conf: std_logic_vector(31 downto 0) := (others => '0');
signal status: std_logic_vector(31 downto 0) := (others => '0');
signal pause_evt_bldr: std_logic := '0';
-- slink signals
signal slink64: std_logic := '0';
signal slink_ctrl: std_logic := '0'; -- marks BOE and EOE
signal slink_down: std_logic := '0';
signal slink_full: std_logic := '0';
signal en_slink: std_logic := '0';
signal slink_rst: std_logic := '0';
signal slink_test: std_logic := '0';
signal slink_we: std_logic := '0';
signal slink_data: std_logic_vector(35 downto 0) := (others => '0');
signal ddr_w_gnt: std_logic := '0';
signal ddr_w_davp: std_logic := '0';
signal ddr_w_datap: std_logic_vector(31 downto 0) := (others => '0');
signal replace_crc: std_logic := '0';
signal ddr_w_dav: std_logic := '0';
signal ddr_w_req: std_logic := '0';
signal ddr_w_addr: std_logic_vector(20 downto 0) := (others => '0');
signal ddr_w_length: std_logic_vector(5 downto 0) := (others => '0');
signal ddr_w_data: std_logic_vector(31 downto 0) := (others => '0');
signal htr_buf_gnt: std_logic := '0';
signal htr_buf_re: std_logic := '0';
signal htr_buf_req: std_logic := '0';
signal htr_buf_full: std_logic_vector(14 downto 0) := (others => '0');
signal htr_buf_fullp: std_logic_vector(8 downto 0) := (others => '0');
signal htr_buf_addr: std_logic_vector(14 downto 0) := (others => '0');
signal htr_buf_length: std_logic_vector(5 downto 0) := (others => '0');
signal htr_buf_data: std_logic_vector(31 downto 0) := (others => '0');
signal mon_buf_empty: std_logic := '0';
signal mon_buf_full: std_logic := '0';
signal mon_buf_afull: std_logic := '0';
signal mon_avl: std_logic := '0';
signal mon_we: std_logic := '0';
signal mon_re: std_logic := '0';
signal mon_wp: std_logic_vector(6 downto 0) := (others => '0');
signal mon_data: std_logic_vector(33 downto 0) := (others => '0');
signal catch_offset: std_logic := '0';
signal lrb1_avl: std_logic := '0';
signal lrb2_avl: std_logic := '0';
signal lrb1_we: std_logic := '0';
signal lrb2_we: std_logic := '0';
signal lrb1_re: std_logic := '0';
signal lrb2_re: std_logic := '0';
signal src_sel: std_logic_vector(1 downto 0) := (others => '0');
signal lrb_data: std_logic_vector(33 downto 0) := (others => '0');
signal lrb1_data: std_logic_vector(35 downto 0) := (others => '0');
signal lrb2_data: std_logic_vector(35 downto 0) := (others => '0');
-- memory test signals
signal mem_test : std_logic := '0';
signal mem_fail : std_logic := '0';
signal chk_done : std_logic := '0';
signal mem_done : std_logic := '0';
-- led display signals
signal leds: std_logic_vector(23 downto 0) := (others => '0');
signal errs: std_logic_vector(23 downto 0) := (others => '0');
signal htr_status: std_logic_vector(14 downto 0) := (others => '0');
signal htr_error: std_logic_vector(14 downto 0) := (others => '0');
signal slink_send: std_logic := '0';
signal display_err: std_logic := '0';
-- error handling
signal err_state: std_logic_vector(1 downto 0) := (others => '0');
signal wr_stateLUT: std_logic := '0';
signal rd_stateLUT: std_logic := '0';
signal wr_mmLUT: std_logic := '0';
signal rd_mmLUT: std_logic := '0';
signal HTR_ow: std_logic := '0';
signal HTR_bz: std_logic := '0';
-- debugging signals
signal rd_debug: std_logic := '0';
signal debug: std_logic_vector(31 downto 0) := (others => '0');
begin
-- fast monitoring signals
process(mclk)
begin
  if(mclk'event and mclk = '1')then
    if(strobe = '1')then
	   if(resync_done = '1' or run = '0')then
	     ttc_resync <= '0';
      elsif(ttc_soft_reset = '1')then
	     ttc_resync <= '1';
	   end if;
		rst_ddr_pa <= reset or wr_scale;
	   if(ttc_resync = '0' or conf(13) = '1')then
	     oc_cntr <= "0000";
	   elsif(inc_oc = '1')then
	     oc_cntr <= oc_cntr + 1;
	   end if;
	   if(oc_cntr = "1010" or (conf(13) = '1' and ttc_soft_resetp = '1'))then
	     resync_done <= '1';
	   else
	     resync_done <= '0';
	   end if;
	   if(slink_we = '1' or ttc_resync = '0')then
	     dcc_quiet <= '0';
	   elsif(inc_oc = '1')then
	     dcc_quiet <= '1';
	   end if;
	   if((conf(13) = '0' and oc_cntr(3) = '0' and ttc_resync = '1') or
	      (conf(13) = '1' and dcc_quiet = '1' and inc_oc = '1'))then
	     ttc_soft_resetp <= '1';
	   else
	     ttc_soft_resetp <= '0';
	   end if;
		if(catch_offset = '0')then
		  ddr_pa_off <= ddr_pa;
		else
		  ddr_pa_off <= ddr_pa + mon_wp;
		end if;
	 end if;
	 if(reset = '1')then
	   status(31) <= '0';
	 elsif(oc_cntr(3) = '1' and (lrb1_we ='1' or lrb2_we = '1'))then
	   status(31) <= '1';
	 end if; 
	 rate_OFW_q <= rate_OFW;
	 inc_rate_OFW <= not rate_OFW_q and rate_OFW;
	 rate_OFWp <= rate_OFW and conf(15);
    if(reset = '1')then
	   state <= "0100";
	 elsif(strobe = '1')then
	   if(conf(12) = '1')then
		  state <= conf(11 downto 8);
	   elsif(run = '0')then
		  state <= "1111";
	   elsif(ttc_resync = '1' or status(31) = '1')then
		  state <= "0100";
		else
        case state is
		    when "1000" =>
	         if(err_state = "11")then
		        state <= "1111";
	         elsif(err_state = "10")then
		        state <= "1100";
	         elsif(sync_lost = '1' or err_state = "01")then
		        state <= "0010";
	         elsif(ovfl_warning = '1' or HTR_ow = '1' or evn_fifo_full = '1' or HTR_bz = '1' or rate_OFWp = '1')then
		        state <= "0001";
				end if;  
		    when "0001" =>
	         if(err_state = "11")then
		        state <= "1111";
	         elsif(err_state = "10")then
		        state <= "1100";
	         elsif(sync_lost = '1' or err_state = "01")then
		        state <= "0010";
	         elsif(evn_fifo_full = '1' or HTR_bz = '1' or rate_OFWp = '1')then
		        state <= "0100";
	         elsif(ovfl_warning = '0' and HTR_ow = '0' and rate_OFWp = '0')then
		        state <= "1000";
				end if;  
		    when "0100" =>
	         if(err_state = "11")then
		        state <= "1111";
	         elsif(err_state = "10")then
		        state <= "1100";
	         elsif(sync_lost = '1' or err_state = "01")then
		        state <= "0010";
	         elsif(evn_fifo_full = '0' and HTR_bz = '0' and rate_OFWp = '0')then
		        state <= "0001";
				end if;  
			 when others => null;
		  end case;
		end if;
	 end if;
  end if;
end process;
lvds_pads.ovfl <= state(0);
lvds_pads.sync <= state(1);
lvds_pads.busy <= state(2);
lvds_pads.ready <= state(3);
--conf(31 downto 16) <=  x"2c21"; -- version number
conf(31 downto 16) <=  version; -- version number
------------------------------------------------------------------------------
-- module instantiation
I_clocks: clocks PORT MAP(
		reset => reset_dll,
		n_strobe => n_strobe,
		mclk => mclk,
		mclkn => mclkn,
		mclk90 => mclk90,
		mclk270 => mclk270,
		ttc_clk => ttc_clk,
		lock => lock,
		clk_pads => clk_pads
	);
not_locked <= not(lock(0) and lock(1) and lock(2) and lock(3));
I_pci3_if: pci3
  port map(
      clk => mclk,
      reset => reset,
		reset_err => reset_err,
      not_locked => not_locked,
 		run => run,
		mem_test => mem_test,
		ddr_w_gnt => ddr_w_gnt,
		chk_done => chk_done,
		mem_done => mem_done,
		addr => addr,
      w_data => w_data,
      r_data => r_data,
		pci3_cycle => pci3_cycle,
      is_high => is_high,
		n_strobe => n_strobe,
      wr_config => wr_config,
      rd_config => rd_config,
		wr_scale => wr_scale,
		rd_scale => rd_scale,
		wr_timeout => wr_timeout,
		rd_timeout => rd_timeout,
		wr_src_id => wr_src_id,
		rd_src_id => rd_src_id,
		rd_mon_wc => rd_mon_wc,
		ttc_soft_reset => ttc_soft_resetp,
		rst_ttc => rst_ttc,
		wr_ttc_id => wr_ttc_id,
		rd_ttc_id => rd_ttc_id,
		wr_bcn_off => wr_bcn_off,
		rd_bcn_off => rd_bcn_off,
		wr_i2c => wr_i2c,
		rd_i2c => rd_i2c,
		rd_evn_pntr => rd_evn_pntr,
		rd_evn_buf => rd_evn_buf,
		rd_CRC_error => rd_CRC_error,
		rd_lrb_sgl_err => rd_lrb_sgl_err,
		rd_lrb_dbl_err => rd_lrb_dbl_err,
		rd_evn_mm => rd_evn_mm,
		rd_counters => rd_counters,
		rd_status_cntr => rd_status_cntr,
		wr_stateLUT => wr_stateLUT,
		rd_stateLUT => rd_stateLUT,
		wr_mmLUT => wr_mmLUT,
		rd_mmLUT => rd_mmLUT,
		rd_lrb_en => rd_lrb_en,
		monitor_start => pci3_LRB_stat,
		rd_lrb_mon_pci1 => rd_lrb_mon_pci1,
		rd_lrb_mon_pci2 => rd_lrb_mon_pci2,
		slink_rst => slink_rst,
		wr_slink_data => wr_slink_data,
		rd_slink_data => rd_slink_data,
		rd_slink_pntr => rd_slink_pntr,
      wr_ddr_data => wr_ddr_data,
      rd_ddr_data => rd_ddr_data,
		cache_chk => cache_chk,
		cache_hit => cache_hit,
		flush_cache => flush_cache,
		rd_ddr_pa => rd_ddr_pa,
		ld_ddr_pa => ld_ddr_pa,
		inc_ddr_pa => inc_ddr_pa,
      rd_status => rd_status,
		pci3_pads => pci3_pads,
		rd_debug => rd_debug
      );
I_ddr_w_queue: ddr_w_queue PORT MAP(
		reset => reset,
		mem_test => mem_test,
		clk => mclk,
		w_addr => w_addr,
		event_avl => mon_avl,
		lrb1_avl => lrb1_avl,
		lrb2_avl => lrb2_avl,
		event_we => mon_we,
		lrb1_we => lrb1_we,
		lrb2_we => lrb2_we,
		event_re => mon_re,
		lrb1_re => lrb1_re,
		lrb2_re => lrb2_re,
		src_sel => src_sel,
		lrb_data => lrb_data,
		event_data => mon_data,
		ddr_w_gnt => ddr_w_gnt,
		ddr_w_dav => ddr_w_davp,
		ddr_w_req => ddr_w_req,
		ddr_w_length => ddr_w_length,
		ddr_w_addr => ddr_w_addr,
		ddr_w_data => ddr_w_datap,
		rd_debug => rd_debug,
		r_data => r_data,
		w_data => w_data,
		addr => addr,
		run => run,
		is_high => is_high,
		n_strobe => n_strobe,
		pci3_cycle => pci3_cycle,
		wr_ddr_data => wr_ddr_data,
		ddr_pa => ddr_pa
	);
process(mclk)
begin
  if(mclk'event and mclk = '1')then
    if(replace_crc = '1')then
	   ddr_w_data <= "0000" & lrb_data(10 downto 0) & '0' & ddr_w_datap(15 downto 0);
	 else
	   ddr_w_data <= ddr_w_datap;
    end if;
	 ddr_w_dav <= ddr_w_davp;
    if(src_sel(0) = '0')then
		replace_crc <= lrb1_we and lrb1_data(32) and not lrb1_data(18);
	 else
		replace_crc <= lrb2_we and lrb2_data(32) and not lrb2_data(18);
	 end if;
  end if;
end process;
--replace_crc <= '0';
I_sdram_ctl: sdram_ctl
    port map(
      reset => reset,
		mem_test => mem_test,
		mem_done => mem_done,
		mem_fail => mem_fail,
		chk_done => chk_done,
		rd_debug => rd_debug,
      clk => mclk,
      clkn => mclkn,
      clk90 => mclk90,
      clk270 => mclk270,
      not_locked => not_locked,
      r_data => r_data,
		addr => addr,
		n_strobe => n_strobe,
		flush_cache => flush_cache,
		rd_ddr_data => rd_ddr_data,
		cache_chk => cache_chk,
		cache_hit => cache_hit,
		ddr_pa => ddr_pa_off,
		ddr_w_gnt => ddr_w_gnt,
		ddr_wfifo_we => ddr_w_dav,
		ddr_w_req => ddr_w_req,
		ddr_w_length => ddr_w_length,
		ddr_w_addr => ddr_w_addr,
		ddr_w_data => ddr_w_data,
		htr_buf_req => htr_buf_req,
		htr_buf_gnt => htr_buf_gnt,
		htr_buf_addr => htr_buf_addr,
		htr_buf_length => htr_buf_length,
		htr_buf_re => htr_buf_re,
		htr_buf_data => htr_buf_data,
		ram_pads => ram_pads
    );
slink_ctrl <= conf(7) when run = '0' else slink_data(34);
I_slink: slink PORT MAP(
		clk => mclk,
		reset => slink_rst,
		n_strobe => n_strobe,
		rd_slink_pntr => rd_slink_pntr,
		rd_slink_data => rd_slink_data,
		wr_slink_data => wr_slink_data,
		addr => addr,
		w_data => w_data,
		r_data => r_data,
		en_slink => en_slink,
		slink64 => slink64,
		slink_ctrl => slink_ctrl,
		slink_din => slink_data,
		slink_we => slink_we,
		slink_full => slink_full,
		slink_down => slink_down,
		inc_slink_wc => inc_slink_wc,
		inc_slink_bc => inc_slink_bc,
		slink_send => slink_send,
		slink_test => slink_test,
		slink_pads => slink_pads
	);
htr_buf_fullp <= htr_buf_full(5 downto 0) & "111";
monitor_start <= pci3_LRB_stat or ttc_LRB_stat;
I_pci1_if: lrb_if PORT MAP(
		clk => mclk,
		reset => reset,
		ttc_reset => ttc_soft_resetp,
		n_strobe => n_strobe,
		run => run,
		htr_buf_full => htr_buf_fullp,
		dav => lrb1_we,
		out_gnt => lrb1_re,
		out_rqst => lrb1_avl,
		lrb_out => lrb1_data,
		auto_monitor => auto_monitor,
		monitor_start => monitor_start,
		lrb_mon_ready => pci1_mon_ready,
		rd_lrb_mon => rd_lrb_mon_pci1,
		lrb_en => lrb_en(8 downto 0),
		addr => addr,
		r_data => r_data,
		lrb_pads => pci1_pads
	);
I_pci2_if: lrb_if PORT MAP(
		clk => mclk,
		reset => reset,
		ttc_reset => ttc_soft_resetp,
		n_strobe => n_strobe,
		run => run,
		htr_buf_full => htr_buf_full(14 downto 6),
		dav => lrb2_we,
		out_gnt => lrb2_re,
		out_rqst => lrb2_avl,
		lrb_out => lrb2_data,
		auto_monitor => auto_monitor,
		monitor_start => monitor_start,
		lrb_mon_ready => pci2_mon_ready,
		rd_lrb_mon => rd_lrb_mon_pci2,
		lrb_en => lrb_en(17 downto 9),
		addr => addr,
		r_data => r_data,
		lrb_pads => pci2_pads
	);
I_ttc_if: ttc_if PORT MAP(
		ttc_clk => ttc_clk,
		clk => mclk,
		reset => reset,
		run => run,
		n_strobe => n_strobe,
		rst_ttc => rst_ttc,
		wr_ttc_id => wr_ttc_id,
		rd_ttc_id => rd_ttc_id,
		wr_bcn_off => wr_bcn_off,
		rd_bcn_off => rd_bcn_off,
		wr_i2c => wr_i2c,
		rd_i2c => rd_i2c,
		r_data => r_data,
		w_data => w_data,
		addr => addr,
		en_brcst => en_brcst,
		ttc_start => ttc_start,
		ttc_stop => ttc_stop,
		ttc_soft_reset => ttc_soft_reset,
		ttc_LRB_stat => ttc_LRB_stat,
		ttc_ready => ttc_ready,
		ttc_serr => ttc_serr,
		ttc_derr => ttc_derr,
		ttc_bcnt_err => ttc_bcnt_err,
		sync_lost => sync_lost,
		rate_OFW => rate_OFW,
		inc_oc => inc_oc,
		inc_l1ac => inc_l1ac,
		inc_bcnterr => inc_bcnterr,
		inc_serr => inc_serr,
		inc_derr => inc_derr,
		evn_fifo_full => evn_fifo_full,
		event_number_avl => event_number_avl,
		event_number => event_number,
		state => state,
		ttc_pads => ttc_pads
	);
I_monitor: monitor PORT MAP(
		reset => reset,
		clk => mclk,
		run => run,
		w_data => w_data,
		r_data => r_data,
		n_strobe => n_strobe,
		wr_scale => wr_scale,
		rd_scale => rd_scale,
		mon_buf_empty => mon_buf_empty,
		mon_buf_full => mon_buf_full,
		mon_buf_afull => mon_buf_afull,
		ddr_pa => ddr_pa_off,
		inc_ddr_pa => inc_ddr_pa,
		rd_mon_wc => rd_mon_wc,
		mon_re => mon_re,
		mon_avl => mon_avl,
		mon_we => mon_we,
		mon_data => mon_data,
		mon_wp => mon_wp,
		catch_offset => catch_offset,
		inc_mon_cntr => inc_mon_cntr,
		slink_we => slink_we,
		slink_data => slink_data
	);
lrb_enp(17) <= lrb_en(15);
lrb_enp(16) <= lrb_en(16);
lrb_enp(15) <= lrb_en(17);
lrb_enp(14) <= lrb_en(12);
lrb_enp(13) <= lrb_en(13);
lrb_enp(12) <= lrb_en(14);
lrb_enp(11) <= lrb_en(9);
lrb_enp(10) <= lrb_en(10);
lrb_enp(9) <= lrb_en(11);
lrb_enp(8) <= lrb_en(6);
lrb_enp(7) <= lrb_en(7);
lrb_enp(6) <= lrb_en(8);
lrb_enp(5) <= lrb_en(3);
lrb_enp(4) <= lrb_en(4);
lrb_enp(3) <= lrb_en(5);
lrb_enp(2 downto 0) <= "000";
I_evt_bldr: evt_bldr PORT MAP(
		reset => reset,
		reset_err => reset_err,
		clk => mclk,
		version => version,
		ovfl_warning => ovfl_warning,
		evn_fifo_full => evn_fifo_full,
		event_number_avl => event_number_avl,
		event_number => event_number,
		lrb_en => lrb_enp,
		src_sel => src_sel,
		lrb1_we => lrb1_we,
		lrb2_we => lrb2_we,
		lrb1_data => lrb1_data,
		lrb2_data => lrb2_data,
		lrb_data => lrb_data,
		w_addr => w_addr,
		mon_wp => mon_wp,
		addr => addr,
		w_data => w_data,
		r_data => r_data,
		run => run,
		n_strobe => n_strobe,
		is_high => is_high,
		rd_evn_pntr => rd_evn_pntr,
		rd_evn_buf => rd_evn_buf,
		wr_timeout => wr_timeout,
		rd_timeout => rd_timeout,
		wr_src_id => wr_src_id,
		rd_src_id => rd_src_id,
		rd_CRC_error => rd_CRC_error,
		rd_lrb_sgl_err => rd_lrb_sgl_err,
		rd_lrb_dbl_err => rd_lrb_dbl_err,
		rd_evn_mm => rd_evn_mm,
		rd_counters => rd_counters,
		rd_status_cntr => rd_status_cntr,
		wr_stateLUT => wr_stateLUT,
		rd_stateLUT => rd_stateLUT,
		wr_mmLUT => wr_mmLUT,
		rd_mmLUT => rd_mmLUT,
		HTR_ow => HTR_ow,
		HTR_bz => HTR_bz,
		err_state => err_state,
		inc_l1ac => inc_l1ac,
		inc_bcnterr => inc_bcnterr,
		inc_oc => inc_oc,
		inc_serr => inc_serr,
		inc_derr => inc_derr,
		inc_rate_OFW => inc_rate_OFW,
		inc_mon_cntr => inc_mon_cntr,
		inc_slink_wc => inc_slink_wc,
		inc_slink_bc => inc_slink_bc,
		htr_status => htr_status,
		htr_error => htr_error,
		htr_buf_gnt => htr_buf_gnt,
		htr_buf_req => htr_buf_req,
		htr_buf_addr => htr_buf_addr,
		htr_buf_length => htr_buf_length,
		htr_buf_re => htr_buf_re,
		htr_buf_data => htr_buf_data,
		htr_buf_full => htr_buf_full,
		TTS => state,
		slink_full => pause_evt_bldr,
		slink_we => slink_we,
		slink_data => slink_data,
		rd_debug => rd_debug
	);
process(mclk)
begin
  if(mclk'event and mclk = '1')then
	 pause_evt_bldr <= slink_full or (mon_buf_afull and conf(14));
  end if;
end process;
i_led_driver: led_driver PORT MAP(
		clk => mclk,
		strobe => strobe,
		err_in => errs,
		leds => leds,
		display_err => display_err,
		led_pads => led_pads
	);
leds(0) <= pci3_cycle;
leds(1) <= ttc_ready;
leds(2) <= inc_l1ac;
leds(3) <= slink_send;
leds(4) <= run;
leds(5) <= state(3);
leds(6) <= state(2);
leds(7) <= state(0);
leds(8) <= state(1);
leds(23 downto 9) <= htr_status;
errs(0) <= '0';
errs(1) <= inc_bcnterr or inc_serr or inc_derr;
errs(2) <= '0';
errs(3) <= slink_down and en_slink;
errs(4) <= '0';
errs(8 downto 5) <= "0000";
errs(23 downto 9) <= htr_error;
------------------------------------------------------------------------------
run <= conf(0);
-- slink reset flag if slink not enabled or received a reset command
en_slink <= conf(1);
slink_test <= conf(2);
slink64 <= not conf(3);
auto_monitor <= conf(4);
en_brcst <= conf(5);
conf(6) <= mem_test;
------------
process(mclk)
begin
  if(mclk'event and mclk = '1')then
    if(strobe = '1')then
--	   conf(0) <= (ttc_start or (wr_config and not is_high and w_data(0)) or conf(0))
--	                and not(ttc_stop or (wr_config and is_high and w_data(0)));
	   conf(0) <= ((wr_config and not is_high and w_data(0)) or conf(0))
	                and not(wr_config and is_high and w_data(0));
-- memory test only when run is stopped
		mem_test <= not run and not mem_fail and not mem_done and 
		            not (wr_config and is_high and w_data(6)) and
		           (mem_test or (wr_config and not is_high and w_data(6)));
-- TTS test
		conf(12) <= not run and not (wr_config and is_high and w_data(12)) and
		           (conf(12) or (wr_config and not is_high and w_data(12)));
	   if(wr_config = '1')then
        if(is_high = '1')then
          conf(15 downto 13) <= conf(15 downto 13) and not w_data(15 downto 13);
          conf(11 downto 7) <= conf(11 downto 7) and not w_data(11 downto 7);
          conf(5 downto 1) <= conf(5 downto 1) and not w_data(5 downto 1);
        else
          conf(15 downto 13) <= conf(15 downto 13) or w_data(15 downto 13);
          conf(11 downto 7) <= conf(11 downto 7) or w_data(11 downto 7);
          conf(5 downto 1) <= conf(5 downto 1) or w_data(5 downto 1);
        end if;
		end if;
    end if;
  end if;
end process;
process(mclk)
begin
  if(mclk'event and mclk = '1')then
    strobe <= n_strobe;
  end if;
end process;
i_pa: l_cntr
  generic map(N=> 7)
  port map(
    d => w_data(6 downto 0),
    load => ld_ddr_pa,
    reset => rst_ddr_pa,
    set => '0',
    ec => strobe,
    en_cnt => inc_ddr_pa,
    clk => mclk,
    q => ddr_pa
  );
pa <= x"000000" & '0' & ddr_pa;
r_data <= pa when rd_ddr_pa = '1' else (others => 'Z');
status(0) <= slink_down;
status(1) <= slink_full;
status(2) <= mon_buf_full;
status(3) <= mon_buf_empty;
--status(4) <= '0';
process(mclk)
begin
  if(mclk'event and mclk = '1')then
    if(reset = '1')then
	   status(4) <= '0';
	 elsif((lrb1_data(33 downto 32) = "11" and lrb1_we = '1') or (lrb2_data(33 downto 32) = "11" and lrb2_we = '1'))then
	   status(4) <= '1';
    end if;
  end if;
end process;
status(5) <= not ttc_ready;
status(6) <= ttc_bcnt_err;
status(7) <= ttc_serr;
status(8) <= ttc_derr;
status(9) <= state(1);
status(10) <= not pci1_mon_ready;
status(11) <= not pci2_mon_ready;
process(mclk)
begin
  if(mclk'event and mclk = '1')then
    status(12) <= display_err or (status(12) and not (rd_status and strobe));
  end if;
end process;
status(13) <= state(0);
status(14) <= state(2);
status(15) <= not state(3);
status(30 downto 16) <= htr_buf_full;
r_data <= status when rd_status = '1' else (others => 'Z');
lrb_en(31 downto 18)	<= (others => '0');
r_data <= lrb_en when rd_lrb_en = '1' else (others => 'Z');
r_data <= conf when rd_config = '1' else (others => 'Z');
-- debug
debug(31 downto 0) <= x"00000000";-- & '0' & mon_buf_full & mon_buf_empty & catch_offset & '0' & ddr_pa_off & '0' & mon_wp;
r_data <= debug when rd_debug = '1' and addr(2 downto 0) = "010" else (others => 'Z');
end dcc_arch;
